#!/bin/bash
#
# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018, 2019. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************
#

if [ "${LIB_SOFTWARE_CHECKS_SOURCED}" = "" ] ; then
LIB_SOFTWARE_CHECKS_SOURCED="true"

##############################################################################
# Verify the 'snapml' is installed on each node. For now do this by checking that
# 'snaprun' is executable on each node.
# Globals:
#   ${VERBOSE}
#   ${WORKING_DIR}
#   ${MPI_ARGS}
# Arguments:
#   Host List
# Returns:
#   None
##############################################################################
verifyHostSoftware()
{
  local HOSTS="${1}"
  local EXECUTABLE="${2}"
  local SPLIT_HOSTS=($(echo ${HOSTS} | tr "," " "))
  local ENCOUNTERED_ERROR="false"
  for ((i=0; i<${#SPLIT_HOSTS[@]}; i++)); do
    # Check if pai4sk is installed
    mpirun -timeout "${DEBUG_TIMEOUT}" "${MPI_ARGS[@]}" -n 1 -H ${SPLIT_HOSTS[i]} python -c "import pai4sk" > "${WORKING_DIR}/${SPLIT_HOSTS[i]}_software.out" 2> "${WORKING_DIR}/${SPLIT_HOSTS[i]}_software.err"
    local ret_code=$?
    if [ ${ret_code} -ne 0 ]; then
      ENCOUNTERED_ERROR="true"
      print_error "ERROR" $MSG_SW_ERROR
      [ "${VERBOSE}" = "true" ] && >&2 cat "${WORKING_DIR}/${SPLIT_HOSTS[i]}_software.err"
      continue
    fi

    # Check if executable is runnable
    mpirun -timeout "${DEBUG_TIMEOUT}" "${MPI_ARGS[@]}" -n 1 -H ${SPLIT_HOSTS[i]} bash -c '[ -x "$(which '${EXECUTABLE}')" ]' > "${WORKING_DIR}/${SPLIT_HOSTS[i]}_executable.out" 2> "${WORKING_DIR}/${SPLIT_HOSTS[i]}_executable.err"
    local ret_code=$?
    if [ ${ret_code} -ne 0 ]; then
      ENCOUNTERED_ERROR="true"
      print_error "ERROR" $MSG_NO_EXECUTABLE
      [ "${VERBOSE}" = "true" ] && >&2 cat "${WORKING_DIR}/${SPLIT_HOSTS[i]}_executable.err"
      continue
    fi
  done

  if [ ${ENCOUNTERED_ERROR} = "true" ]; then
    exit 1
  fi
}

fi
