# *****************************************************************
#
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2018. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# *****************************************************************

from scipy import sparse

# Load the data
import time
import numpy as np
import argparse

from sklearn.metrics import accuracy_score

defaultPath = "."
CLI=argparse.ArgumentParser()

CLI.add_argument(
    "--skip_sklearn",
    action = 'store_true',
    default=False
)
CLI.add_argument(
    "--data_path",
    type=str,
    default=defaultPath
)

args = CLI.parse_args()

t0 = time.time()
X_train = np.load(args.data_path + "/data/SUSY.X_train.npy")
X_test  = np.load(args.data_path + "/data/SUSY.X_test.npy")
y_train = np.load(args.data_path + "/data/SUSY.y_train.npy")
y_test  = np.load(args.data_path + "/data/SUSY.y_test.npy")
print("Data load time (s):  {0:.2f}".format(time.time()-t0))

np.random.seed(44)
random_state=42

# Enable for regression with sample weighting
# sample_weight = np.random.randint(low=1, high=10, size=X_train.shape[0])
sample_weight = None

max_depth = 12
n_estimators = 10
n_jobs = 10
max_features = 4

# Import snap RandomForestClassifier from pai4sk module directly
from pai4sk import RandomForestClassifier as SnapForest
rf = SnapForest(random_state=random_state, max_depth=max_depth, n_estimators=n_estimators, n_jobs=n_jobs, max_features=max_features)

# Training time
t0 = time.time()
rf.fit(X_train, y_train, sample_weight=sample_weight)
print("[snap] Training time (s):  {0:.2f}".format(time.time()-t0))
       
# Inference
pred_test = rf.predict(X_test)
acc_snap = accuracy_score(y_test, pred_test)
print("[snap] Accuracy score:   {0:.4f}".format(acc_snap))


if not args.skip_sklearn:    
    # Import sklearn's RandomForestClassifier from pai4sk module directly
    from pai4sk.ensemble import RandomForestClassifier as SklearnForest

    rf = SklearnForest(random_state=random_state, max_depth=max_depth, n_estimators=n_estimators, n_jobs=n_jobs, max_features=max_features)

    # Training time
    t0 = time.time()
    rf.fit(X_train, y_train, sample_weight=sample_weight)
    print("[sklearn] Training time (s):  {0:.2f}".format(time.time()-t0))

    # Inference
    pred_test = rf.predict(X_test)

    # Evaluate accuracy_score on test set
    acc_sklearn = accuracy_score(y_test, pred_test)
    print("[sklearn] Accuracy score:   {0:.4f}".format(acc_sklearn))


max_depth = 6

# pai4sk snap random forest with histograms
rf = SnapForest(random_state=random_state, max_depth=max_depth, use_histograms=True, hist_nbins=64, n_estimators=n_estimators, n_jobs=n_jobs, max_features=max_features)

# Training time
t0 = time.time()
rf.fit(X_train, y_train, sample_weight=sample_weight)
print("[snap.histogram.depth6] Training time (s):  {0:.2f}".format(time.time()-t0))

# Inference
pred_test = rf.predict(X_test)
acc_snap = accuracy_score(y_test, pred_test)
print("[snap.histogram.depth6] Accuracy score:   {0:.4f}".format(acc_snap))

if not args.skip_sklearn:
    rf = SklearnForest(random_state=random_state, max_depth=max_depth, n_estimators=n_estimators, n_jobs=n_jobs, max_features=max_features)

    # Training time
    t0 = time.time()
    rf.fit(X_train, y_train, sample_weight=sample_weight)
    print("[sklearn.depth6] Training time (s):  {0:.2f}".format(time.time()-t0))

    # Inference
    pred_test = rf.predict(X_test)

    # Evaluate accuracy_score on test set
    acc_sklearn = accuracy_score(y_test, pred_test)
    print("[sklearn.depth6] Accuracy score:   {0:.4f}".format(acc_sklearn))

